# -*- coding: utf-8 -*-

from abc import abstractmethod, ABCMeta
from csst_ifs_common.common import CsstIFSInit
from csst_ifs_common.ifs_status import CsstIFSStatus
from csst_ifs_common.ifs_result import CsstIFSResult
from csst_shao_common.interface import BasePipelineInterface

csst_ifs = CsstIFSInit()


class BasePipelineInterface(metaclass=ABCMeta):
    @abstractmethod
    def run(self):
        """run the pipeline"""
        pass

    @abstractmethod
    def para_set(self, *args):
        pass

    @abstractmethod
    def result_check(self):
        pass

    def loger(func) -> typeing.Callable:
        def inner(*args, **kwargs):
            try:
                result = func(*args, **kwargs)
                csst_mci.logger.info(f"log: {func.__name__}{args}->{result}")
                return result
            except Exception as e:
                csst_mci.logger.error(f"error: {func.__name__}->{str(e)}")

        return inner


# 数据处理软件编写主调用类范例
class PipelineL1MCIInstrument(BasePipelineInterface):
    # 此处对程序进行概要介绍, 对类的属性(Attributes)、方法(Methods)等进行一一介绍。
    """
    Instrument is xxxx

    ...

    Attributes
    ----------
    obsid : str
        The obsid of this item.

    logger :
        Inheritance the Logging package for Python. Get a logger for CSST L1 Pipeline.

    self.status = CsstResult

    Methods
    -------
    para_set(image_path: str = "/path/to/image",
             weight_path: str = "/path/to/weight",
             flag_path: str = "/path/to/flag",
             cat_ref_path: str = "/path/to/cat_ref",
             deg: int = 2,
             ) -> CsstResult

        set para

    run()
        run the pipeline

    result_check()
        check the result

    """

    def __init__(self):
        self.result = CsstStatus
        self.logger = csst_mci.logger
        self.status = None
        self.obsid = None

        # 在此处编写代码，使得用户通过执行此方法运行程序
    def run(self):
        """Method, run the pipeline.

        Parameters
        ----------
        # None

        Returns
        -------
        CsstResult
            Result containing `status`, `file_list`, and `output`.
        """

        from csst_mci_instrument.top_level_interface import inst_corr

        # 获取在para_set中配置的参数，此处用于整理，也可以不写，直接喂给需要运行的程序。
        bias_ref = self._bias_ref
        dark_ref = self._dark_ref
        flat_ref = self._flat_ref
        shutter_ref = self._shutter_ref
        sci_path = self._sci_path
        output_dir = self._output_dir

        # 核心调用代码，用于调用开发者各自开发的程序，其中self.status是个人自定义的返回码，可以以任意形式返回
        self.logger.info("%s running..." % self.__name__)
        self.status = inst_corr(
            sci_path, bias_ref, dark_ref, flat_ref, shutter_ref, output_dir, logger=None, os_bias=False, cr_flag=False)

        # 如果生成结果是内存形式存在的fits头文件的情况。
        fits_header = "xxxxx"

        # 根据程序运行结果，返回成功、警告、或者报错，此处为一种方法，也可以用其他方式返回错误。
        def case_perfect_1():
            status = CsstStatus.PERFECT
            self.logger.info("Pipeline instrument Done!")
            return status

        def case_warning_1():
            status = CsstStatus.WARNING
            self.logger.info("Pipeline instrument something warning, xxxx")
            return status

        def case_error_1():
            status = CsstStatus.ERROR
            self.logger.info("Pipeline instrument something wrong, xxxx")
            return status

        def case_error_2():
            status = CsstStatus.ERROR
            self.logger.info("Pipeline instrument something wrong, xxxx")
            return status

        switch_dict = {
            20: case_perfect_1,
            31: case_warning_1,
            41: case_error_1,
            42: case_error_2,
            "default": case_perfect_1
        }

        status = switch_dict.get(self.status)

        # 最终返回结果
        result = CsstResult(
            status=status,
            files=[
                "/path/to/file.fits",
                "/path/to/file.txt",
            ],
            para_01=1,
            fits_header=fits_header
        )
        return result

    # 在此处编写代码，使得用户通过执行此方法配置流水线运行参数，所有参数应当含有默认值，输入参数非必须。此处可以通过指定参数，给与不同情况的初始化参数配置
    def para_set(self,
                 image_path: str = "/path/to/image",
                 weight_path: str = "/path/to/weight",
                 flag_path: str = "/path/to/flag",
                 cat_ref_path: str = "/path/to/cat_ref",
                 deg: int = 2,
                 ) -> CsstResult:
        """Method, estimate image distortion for a single MSC-MBI image.

        Parameters
        ----------
        image_path: str
            Path of image, defaulting to "/path/to/image".
        weight_path: str
            Path of weight, defaulting to "/path/to/weight".
        flag_path: str
            Path of flag, defaulting to "/path/to/flag".
        cat_ref_path: str
            Path of reference catalog, defaulting to "/path/to/cat_ref".
        deg


        Returns
        -------
        CsstResult
            Result containing `status`, `file_list`, and `output`.
        """
        from csst_mci_common.config_crds import env_crds_set_ins_csst1
        from csst_mci_common.config_dfs import env_dfs_set_ins_csst1
        from csst_mci_common.config_dfs import env_dfs_set_l0_csst1

        env_crds_set_ins_csst1()
        self.obsid = csst_mci.config.get_mci_config("obsid")
        env_dfs_set_l0_csst1(obsid=self.obsid)
        env_dfs_set_ins_csst1(target_detector)

        self._bias_ref = csst_mci.config.get_mci_config("bias_ref")
        self._dark_ref = csst_mci.config.get_mci_config("dark_ref")
        self._flat_ref = csst_mci.config.get_mci_config("flat_ref")
        self._shutter_ref = csst_mci.config.get_mci_config("shutter_ref")
        self._sci_path = csst_mci.config.get_mci_config("sci_path")
        self._output_dir = csst_mci.config.get_mci_config("ins_output")

        return

    # 在此处编写代码，使得用户通过执行此方法配置本地运行所需要的参数，此处方法可用于单元测试或实际单机运行。
    def para_set_local(
            self,
            image_path: str = "/path/to/image",
            bias_ref_path: str = "/path/to/bias",
            weight_path: str = "/path/to/weight",
            flag_path: str = "/path/to/flag",
            cat_ref_path: str = "/path/to/cat_ref",
            deg: int = 2,
            ) -> CsstResult:
        """Method, estimate image distortion for a single MSC-MBI image.

        Parameters
        ----------
        image_path: str
            Path of image, defaulting to "/path/to/image".
        weight_path: str
            Path of weight, defaulting to "/path/to/weight".
        flag_path: str
            Path of flag, defaulting to "/path/to/flag".
        cat_ref_path: str
            Path of reference catalog, defaulting to "/path/to/cat_ref".
        deg


        Returns
        -------
        CsstResult
            Result containing `status`, `file_list`, and `output`.
        """
        self.image_path = image_path
        self.bias_ref = bias_ref_path
        self.dark_ref = csst_mci.config.get_mci_config("dark_ref")
        self.flat_ref = csst_mci.config.get_mci_config("flat_ref")
        self.shutter_ref = csst_mci.config.get_mci_config("shutter_ref")
        self.sci_path = csst_mci.config.get_mci_config("sci_path")
        self.output_dir = csst_mci.config.get_mci_config("ins_output")
        return


    def para_set_custom_1(self):
        return

    # 在此处编写代码，使得用户通过执行此方法进行结果检查
    def result_check(self):
        pass
